import urllib.request
import urllib.error
import json
from bs4 import BeautifulSoup
from hashlib import md5
import copy
proxyHttp = None
import json

def proxySetting(http=None, https=None):
    if http:
        global proxyHttp
        proxyHttp = http
        print ("!!Proxy set to " + str(proxyHttp))



class Container(object):
    def __init__(self, id, url=None, fileName=None):
        if not id:
            raise ValueError("identifier is empty")
        self.id = id
        self.rawData = None
        self.url = url
        self.fileName = fileName

    def getXmlHandler(self):
        if not self.fileName:
            print ('got to fetch ' + self.id)
        self.rawData = self._fetch() if not self.fileName else self._readFile()
        if not self.rawData:
            raise ValueError("Error, empty xmlHandler")
            
        xmlH = BeautifulSoup(self.rawData, 'xml')
        return xmlH

    def _readFile(self):
        with open (self.fileName, "r") as f:
            rawData = f.read()
        return rawData

    def _fetch(self):
        global proxyHttp
        #print ("Trying to DL " + self.url)
        try:
            if proxyHttp:
                #proxy = urllib2.ProxyHandler({'http': proxyHttp})
                proxy_support = urllib.request.ProxyHandler(proxyHttp)
                opener = urllib.request.build_opener(proxy_support)
                urllib.request.install_opener(opener)
                con = urllib.request.urlopen(self.url)
               # soup = BeautifulSoup(''.join(website))
            else:
                #print 'using no proxy'
                #p2.7
                #req = urllib2.Request(self.url, headers={'User-Agent' : "Magic Browser"})
                #con = urlopen( req )
                con = urllib.request.urlopen(self.url)
            rawData =  con.read()
            #response = urllib2.urlopen(self.url)

        except urllib.error.HTTPError as error:
            print ("Error at " + self.url)
            print ("HTTP ERROR " + str(error.code))
            print (error.fp.read())
            return None
        except urllib.error.URLError as error:
            print (self.url)
            print (error.reason)
            return None
        #rawData = response.read()
        #response.close()
        return rawData

    def serialize (self):
        return self.rawData

    @property
    def raw(self):
        if not self.rawData:
            self.rawData = self._fetch() if not self.fileName else self._readFile()
        return self.rawData

'''
class mdTreeEncoder(json.JSONEncoder): # DOESNOT WORK stringifed twice (by encoder too)
    def default(self, mdTreeInstance):
        if isinstance(mdTreeInstance, mdTree):
            d = {}
            for x,y,lv in mdTreeInstance:
                if not x in d:
                    d[x] = {}
                if not y in d[x]:
                    d[x][y] = None
                    try:
                        d[x][y] = [ str(e) for e in lv ]
                    except TypeError:
                        d[x][y] = str(lv)
                   
                return d
        # Error
        return json.JSONEncoder.default(self, mdTreeInstance)
'''


'''
    def jsonString(self, leaveJsonifyer=None):
        uBuff = []
        #print(leaveJsonifyer)
        for x in self.data:
            buf = []
            for y in self.data[x]:
                lv = self.data[x][y]
                ls = '00'
                try:
                    ls = '[' + ','.join( [ '"' + json.dumps(e) + '"' if leaveJsonifyer is None else '"' + leaveJsonifyer(e) + '"' for e in lv ] ) + ']'
                except TypeError:
                    print("Non iterable")
                    ls = json.dumps(lv) if leaveJsonifyer is None else leaveJsonifyer(lv)
                
                buf.append('"' + str(y) + '" : ' + ls)
            uBuff.append( '"'  + str(x) + '" : {' + ",".join(buf) + "}" )
        return  '{"mdTree" : { ' + ','.join(uBuff) + '}}'

'''


## md5 hashed tree
## Semi matrix representation as a 
## Two lvl tree, top/down key are min/max md5 hash
class mdTree:
    def __init__(self, append=True):

        self.autoAppendable = append

       # print(self.autoAppendable)

        self.data = {}
    
    def __len__(self): # total number of terminal leaves == non-empty matrix cells
        c = 0
        for k1 in self.data:
            for k2 in self.data[k1]:
                c+=1
        return c

    def keys(self): # Returns all primary and secondary keys
        return set ( list(self.data.keys()) + [ k for k in self.data for _k in self.data[k] ] )

    def _digest(self, k1, k2):
        _k1 = md5(k1.encode('utf-8')).hexdigest()
        _k2 = md5(k2.encode('utf-8')).hexdigest()
        x = k1 if _k1 < _k2 else k2 
        y = k2 if _k1 < _k2 else k1 
        return x,y

    def append (self, k1, k2, datum):# appendig
        if not self.autoAppendable:
            raise TypeError("Cant append to custom leave")

        x, y = self._digest(k1, k2)
        self._push(x,y, datum)

    def get (self, k1, k2): # Mutable ref returned
        x, y = self._digest(k1, k2)
        if x not in self.data:
            return None
        if y not in self.data[x]:
            return None
        return self.data[x][y]
    
    def getOrSet (self, k1, k2, value): # Mutable ref returned
        x, y = self._digest(k1, k2)
        el = self._getMaySet(x, y, value)
        return el

    def _getMaySet(self, x, y, value): # Initailise leave to value if not created already
        if x not in self.data:
            self.data[x] = {}
        if y not in self.data[x]:
            self.data[x][y] = value
        return self.data[x][y]


    def _testRef(self, x, y):
        if x not in self.data:
            return False
        if y not in self.data[x]:
            return False
        return True
    
   
        
    def _push(self, x, y, datum):
        buf = self._getMaySet(x, y, [])
        buf.append(datum)

# We unroll to copy ref of collection elements not of collection itself
# to avoid collection mutability
    def __getitem__(self, k1):
        data = {}
        if k1 in self.data:
            #data[k1] = {}
            for subk1 in self.data[k1]:
                if self.autoAppendable:
                    data[subk1] = [ e for e in self.data[k1][subk1] ]
                else:
                    data[subk1] = self.data[k1][subk1]
        
        _k1 = md5(k1.encode('utf-8')).hexdigest()
        
        
        for k2 in self.data:
            if _k1 < md5(k2.encode('utf-8')).hexdigest():
                continue
            if k1 in self.data[k2]:
                if self.autoAppendable:
                    data[k2] = [ e for e in self.data[k2][k1] ]
                else:
                    data[k2] = self.data[k2][k1]

        return data

    def __iter__(self):
        for k1 in self.data:
            for k2 in self.data[k1]:
                yield(k1, k2, self.data[k1][k2])

    def dnCopy(self):
        other = dnTree()
        other.data = copy.deepcopy(self.data)
        return other


    def __repr__(self):
        return str(self.data)

## dEnSE Tree
## Minimal number of top level leaves tree
## Wrapper to md5 tree with dynamic access to content
##
class dnTree(mdTree):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.weights = {}
        self._rank = None

    def append(self, k1, k2, datum):
        super(dnTree, self).append(k1, k2, datum)
        for x in (k1, k2):
            if x not in self.weights:
                self.weights[x] = 0
            self.weights[x] += 1
        self._rank = None

    def __getitem__(self, k1):
        #print("allKeys: " + str(super().__getitem__(k1)))
        #print(self.rank)
        return { k2: v for k2,v in super().__getitem__(k1).items() if self.rank[k1] < self.rank[k2] }
    
    # Get All elements connected to k1, w/ weight consideration
    def getNonDense(self, k1):
        try :
            d = super().__getitem__(k1)
            return d
        except KeyError:
            return None
        
    @property 
    def rank(self): # sorting key by number of insertion, and convert into non redundatn rank numbers
        #self._rank = None
        if not self._rank:   
            _rank = sorted(self.weights.keys(), key= lambda k :  self.weights[k], reverse=True)
            #print("RK::", str(_rank))
            self._rank = { k : rk  for rk, k in enumerate(_rank) }
        #print ("-->", self._rank)
        return self._rank

    def __repr__(self):
        d = {}
        for k in super().keys():
            _d = self[k]
            if _d:
                d[k] = _d
        
        return str(d)



import pprint
from Bio import AlignIO
import re
import sys
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.Alphabet import IUPAC
import json
import copy

import pyproteins.alignment.nw_custom
import pyproteins.sequence.peptide
import pyproteins.alignment.scoringFunctions

from multiprocessing import Pool


aaCode = {
        'A':'A',
        'C':'C',
        'D':'D',
        'E':'E',
        'F':'F',
        'G':'G',
        'H':'H',
        'I':'I',
        'K':'K',
        'L':'L',
        'M':'M',
        'N':'N',
        'P':'P',
        'Q':'Q',
        'R':'R',
        'S':'S',
        'T':'T',
        'V':'V',
        'W':'W',
        'Y':'Y',
        'Z': 'E',
        'B': 'D',
        'J': 'L',
        'X': 'A',
        'U': 'C',
        ' ': '-',
        '-': '-'
    }


pp = pprint.PrettyPrinter(indent=4)


class initError(Exception):
    pass

class RulerError(Exception):
    def __init__(self, ruler):
        self.ruler = ruler

class SliceError(Exception):
    def __init__(self, sliceExpression):
        self.sliceExpression = sliceExpression

class AlignmentError(Exception):
    def __init__(self, nSeq):
        self.nSeq = nSeq

class AlignmentLengthError(AlignmentError):
    def __init__(self, base, nSeq, nLen):
        AlignmentError.__init__(self, nSeq)
        self.nSeq = nSeq
        self.nLen = nLen

class AlignmentTypeError(AlignmentError):
    def __init__(self, nSeq, nPos, char):
        AlignmentError.__init__(self, nSeq)
        self.nPos = nPos

def aaCoherce(key):
    global aaCode
    if key in aaCode:
        return aaCode[key]

    print("Warning : weird amino-acid 1 letter code:",key)
    return 'A'


# Quick and dirty quality assesment of the coverage of one sequence w/ respect to the MASTER
# no gap is expected in master ie 'seqOne'
# We dont do a pairwise optimal sequence alignment, just read straight from the msa
def seqPairScores(seqOne, seqTwo):
    needle = pyproteins.alignment.scoringFunctions.Needle()

    w1 = [Position( {'aa' : c, 'ss2' : None, 'burial' : None }) for c in seqOne]
    w2 = [Position( {'aa' : c, 'ss2' : None, 'burial' : None }) for c in seqTwo]

    s_id = 0
    s_sim = 0
    s_cov = 0

    n_count = 0

    for i in range( min(len(seqOne), len(seqTwo)) ):
        if '-' ==  w2[i].aa and '-' ==  w1[i].aa: # Both have gaps, ignore position
            continue

        if '-' != w1[i].aa:
            n_count += 1 # aligned position of reference sequence count it

        if '-' !=  w2[i].aa and '-' !=  w1[i].aa: # reference sequence does not have a gap,
            s_cov += 1

        if '-' ==  w2[i].aa or '-' ==  w1[i].aa:
            continue


        if w1[i].aa == w2[i].aa:
            s_id += 1
            s_sim += 1
            continue
        sc = needle.fScore(w1[i], w2[i])
        if sc > 0:
            s_sim += 1

    #n = float(len(seqOne) - n_miss)
    n = float(n_count)

   # print "--->" + str(n) + " :: " + str(s_sim)
   # print "=>" + str( round(s_sim / n, 2) )

    return (round(s_cov / n, 2),round(s_sim / n, 2), round(s_id / n, 2) )


class Position():  ## Used for NW and custom scoring interface
    def __init__(self, e):
        self.aa = e['aa'].upper()
        self._ss2 = e['ss2']
        self.burial = e["burial"]
    def __repr__(self):
        return self.aa
    def __eq__(self, other):
        if self.aa != other.aa:
            return False
        if self._ss2 and other._ss2:
            if self._ss2 != other._ss2:
                return False
        elif self._ss2 and not other._ss2:
            return False
        elif other._ss2 and not self._ss2:
            return False

        if self.burial and other.burial:
            if self.burial != other.burial:
                return False
        elif self.burial and not other.burial:
            return False
        elif other.burial and not self.burial:
            return False

        return True
    @property # emulate str(self.ss2)
    def ss2(self):
        if self._ss2:
            return self._ss2
        return ' '

class VectorFreq(): # this class provides the interface w/ the nw custom scoring
    def __init__(self, msaObject, seqNumRef=0):
        self.refSeq = msaObject[seqNumRef]['sequence']
        skippedColumns = [i for i,j in enumerate(self.refSeq) if j == '-']
        self.__hookedMsa__ = msaObject
        freq = [[ col[aa] / float(msaObject.shape()[0]) for aa in msaObject.alphabet ] for icol, col in enumerate(msaObject.frequency) if icol not in skippedColumns ]
        self.refSeq = self.refSeq.replace('-','', len(self.refSeq))
        self.data = [freqPosition({'aa' : self.refSeq[i], 'ss2' : None, 'burial' : None, 'f' : freq[i] }, self.__hookedMsa__) for i in range(0, len(freq))]

    def homogenize(self):
        def fn(fPos):
            i = fPos.alphabet.index(fPos.aa)
            fPos.freq = [ 1.0 if i == j else 0.0 for j,f in enumerate(fPos.freq) ]
            return fPos
        self.data = [fn(fPos) for fPos in self.data]


    def __repr__(self):
        return str(self.refSeq)

    def __getitem__(self, i):
        if i < 0:
            raise ValueError("minimal amino-acid number is 1")
        return self.data[i]

    @property
    def alphabet(self):
        return self.__hookedMsa__.alphabet

    @property
    def id(self):
        return self.__hookedMsa__.id

    def __len__(self):
        return len(self.refSeq)

    @property
    def hasSse(self):
        return False

class freqPosition(Position):  ## Used for NW and custom scoring interface
    def __init__(self, e, __hookedMsa__):
        Position.__init__(self, e)
        self.freq = e['f']
        self.__hookedMsa__ =  __hookedMsa__
    #def __repr__
    #def __eq__(i, j):
    def __len__(self):
        return len(self.freq)
    @property
    def alphabet(self):
        return self.__hookedMsa__.alphabet
    def __getitem__(self, k):
        return self.freq[k]

#
#
#
# will return a vector of msaRef column numbers, with a value for the
# eventual corresponding column number in msaShrink
#
# TODO: wrap&use hmmr to align the two msa by their profiles
#
#

def map(msaRef, msaShrink):
    #import scoringFunctions
    #import nw_custom
    PWS=pyproteins.alignment.scoringFunctions.PWS(alphabet=msaRef.alphabet)
    nw = pyproteins.alignment.nw_custom.nw(matchScorer=PWS.fScore)
    aln = nw.align(msaRef.vectors, msaShrink.vectors)

    return aln

# We need to fill a matrix of distances sized to the query msa

#def cloneAlphaDist():



class MsaBean():
    def __init__(self, matrix, header, backtrack=None):
        self.header = header
        self.matrix = [ [aaCoherce(x) for x in r] for r in matrix ]
        self.backtrack = backtrack
        # HARD REPLACE ALL X instances by A


class Msa(object):  # HARD REPLACE ALL X instances by A

    def __len__(self):
        return self.nSeq

    def vectors(self, seqNumRef=0):
        return VectorFreq(self, seqNumRef=0)


    def _scan_parallel_joblib(self, queryPeptide, n):
        Needle=pyproteins.alignment.scoringFunctions.Needle(alphabet=self.alphabet)
        nw_scan = pyproteins.alignment.nw_custom.nw(gapOpen=-10, gapExtend=-0.5, matchScorer=Needle.fScore)
        #mPool = mp.Pool(n)

        inputs = [ [queryPeptide, p] for p in self.peptideArray ]
        
        with Pool(4) as p:
            #print(p.map(f, [1, 2, 3]))
            hits = p.map(nw_scan.mp_align, inputs)
        return [{ 'sim' : float(ali.simPct) ,'ali': ali} for ali in hits]

    def scan(self, queryPeptide=None, _num_worker=1, recordLookup=False):
        #print "Scaning a " + str(self.nSeq) + " sequences msa w/ " + str(_num_worker) + " threads"
        if _num_worker > 1:
            hit_buffer=self._scan_parallel_joblib(queryPeptide, _num_worker)
        else:
            hit_buffer = []
            Needle = pyproteins.alignment.scoringFunctions.Needle(alphabet=self.alphabet)
            nw_scan = pyproteins.alignment.nw_custom.nw(gapOpen=-10, gapExtend=-0.5, matchScorer=Needle.fScore)
            for i in range(0, self.nSeq):
                ali = nw_scan.align(queryPeptide, self.getPept(i))
                hit_buffer.append({ 'sim' : float(ali.simPct) ,'ali': ali})


        return sorted(hit_buffer, key=lambda k: k['sim'])

    @property
    def peptideArray(self):
        for i in range (self.nSeq):
            p = self.getPept(i)
            yield(p)

    def getPept(self, x):
        hits = self.recordLookup(num=x)
        rec = hits.pop(0)
        return pyproteins.sequence.peptide.Entry(id=rec['header'], seq=rec['sequence'])

    def __init__(self, fileName = None, msaBean = None, alphabet = "ACDEFGHIKLMNPQRSTVWY-", backtrack = None, jsBeanFile = None, id=None):
        self.alphabet = alphabet
        self.backtrack = backtrack
        self.id = id if id else "id_default_msa"
        if fileName:
            fType = None
            if fileName.endswith(".aln"): fType = "clustal"
            else : fType = "fasta"
            self.alignment = AlignIO.read(open(fileName), fType) # fasta
            self.asMatrix = [[ aaCoherce(aa) for aa in list(record.seq) ] for record in self.alignment]

            self.headers = [record.id for record in self.alignment]
           # print dir(self.alignment)
           # print dir (self.asMatrix)
           # print self.alignment.annotations
        elif msaBean:
            self.asMatrix = msaBean.matrix
            self.headers = msaBean.header
            self.backtrack = msaBean.backtrack
            #self.alignment = [seq for seq in msaBean.matrix]
            # same for headers
        elif jsBeanFile:
            with open(jsBeanFile) as json_file:
                json_data = json.load(json_file)
            self.asMatrix = json_data['matrix']
            self.headers = json_data['headers']
            self.backtrack = json_data['backtrack']

        else:
            raise initError("You must specify a bean or a mfasta file")

        self.nSeq = len(self.asMatrix)
        self.length = len(self.asMatrix[0])
        self._frequency = None
    # Putting frequency attribute in lazy instantiation leads to a factor x4 constructor time
    @property
    def frequency(self):
        if not self._frequency:
            self._frequency = [{c : 0 for c in list(self.alphabet)} for nCol in range(self.length)]

            for oRow in enumerate(self.asMatrix):

                nRow = oRow[0]
                row = oRow[1]
                try:
                    if (len(row) != self.length):
                        raise AlignmentLengthError(nSeq = nRow, nLen = len(row), base = self.length)
                    for oCell in enumerate(row):
                        nCol = oCell[0]
                        char = oCell[1]
                        if not re.search(char, self.alphabet):
                            print("unknown character", char)
                            raise AlignmentTypeError(nSeq = nRow, nPos = nCol, char = char)
                            #sys.exit()
                        else:
                            self._frequency[nCol][char] += 1

                except AlignmentLengthError as e:
                    print ("Wrong sequence length", \
                        str(e.nLen), "do not match at",\
                        str(e.nSeq), "(base length is", str(e.nLen), ")" \
                    )
                    sys.exit(0)
                except AlignmentTypeError as e:
                    print( "unknown character \'", str(e), "\'at position", str(e.nPos)\
                    + " in sequence", str(e.nSeq) )
                    sys.exit(0)

        return self._frequency

    def __iter__(self):
        for i in range(0, self.nSeq):
            if self.backtrack:
                yield {'header' : self.headers[i], 'seq' : self.asMatrix[i], 'back' : self.backtrack[i]}
            else:
                yield {'header' : self.headers[i], 'seq' : self.asMatrix[i] }
    def set_id(self, id):
        self.id = id

    def setHeader(self, i, string):
        self.headers[i] = string

    def sliceTo(self, litteral=None, columnSlice='*', recordSlice='*'):

        def _parse(expr):
            columnSlice='*'
            recordSlice='*'
            array = expr.split(";") # ['c:<10', 's:[1:5, 7:8]']
            try:
                for exp in array:
                    if exp.startswith('c:'):
                        columnSlice = exp.replace("c:", '', 1)
                    elif exp.startswith('s:'):
                        recordSlice = exp.replace("s:", '', 1)
                    else:
                        raise SliceError
            except(SliceError) as e:
                print("Invalid slice expression \'", litteral, "\'")

            return {
                'column' : columnSlice,
                'record' : recordSlice
            }

        def _factorize (ruler):
            ruleOut = []
            curUp = ruler[0]['up']
            curLo = ruler[0]['lo']
            for ind, bracket in enumerate(ruler):
                if ind == 0:
                    continue
                if bracket['lo'] == (curUp + 1):
                  #  print "FUSE " + str(curUp) +" to " + str(bracket['up'])
                    curUp = bracket['up']
                else:
             #       print ""
                    ruleOut.append({'lo' : curLo, 'up' : curUp})
                    curLo = bracket['lo']
                    curUp = bracket['up']

            ruleOut.append({'lo' : curLo, 'up' : curUp})

            #print "factorized ruler " + str(ruleOut)
            return ruleOut
                #{ 'lo' : int(reg.split(':')[0]), 'up'

        def _setSlicer(sliceExpression, nMax):
            ruler = None
            if isinstance(sliceExpression, list):
                ruler = [ { 'lo' : int(reg.split(':')[0]), 'up' : int(reg.split(':')[1]) } for reg in sliceExpression ]
            elif isinstance(sliceExpression, basestring):
                if sliceExpression == "*":
                    ruler = [{'lo' : 0, 'up' : nMax - 1 }]
                else:
                    try:
                        m = re.search("(<=|<|\?|>|>=){1}(\d+)", sliceExpression)
                        if not m:
                            raise SliceError(sliceExpression)
                        else:
                            if not m.groups()[1]:
                                raise SliceError(sliceExpression)
                        #print m.groups()
                            if m.groups()[0] == "<=":
                                ruler = [{'lo' : 0, 'up' : int(m.groups()[1])}]
                            elif m.groups()[0] == "<":
                                ruler = [{'lo' : 0, 'up' : int(m.groups()[1]) - 1}]
                            elif m.groups()[0] == ">=":
                                ruler = [{'lo' : int(m.groups()[1]), 'up' : nMax - 1}]
                            elif m.groups()[0] == ">":
                                ruler = [{'lo' : int(m.groups()[1]) + 1, 'up' : nMax - 1}]
                            else:
                                raise SliceError(sliceExpression)
                    except SliceError as e:
                        print("Invalid slice expression \'", str(e.sliceExpression), "\'"\
                              + "must be \"*\" or \">=|>|<|<=someNumber\"")
                        sys.exit(0)
            ruleOut = _factorize(ruler)
            return ruleOut

        def _isRuledIn(nSeq, ruler):
            try:
                for interval in ruler:
                    if nSeq >= interval['lo'] and nSeq <= interval['up']:
                        return True
            except(TypeError):
                raise RulerError(ruler=ruler)
            return False

        if litteral:
            data = _parse(litteral)
            columnSlice = data['column']
            recordSlice = data['record']

        seqRuler = _setSlicer(recordSlice, self.nSeq)
        try:
            matrixTmp = [ x for ind, x in enumerate(self.asMatrix) if _isRuledIn(ind, seqRuler)]
            header = [ x for ind, x in enumerate(self.headers) if _isRuledIn(ind, seqRuler)]
        except(RulerError) as e:
            print ("Invalid ruler to splice records of Msa", e.ruler)
            sys.exit(0)
        colRuler = _setSlicer(columnSlice, self.length)
        try:
            matrix = [[ x for ind, x in enumerate(seq) if _isRuledIn(ind, colRuler)] for seq in matrixTmp]
        except(RulerError) as e:
            print("Invalid ruler to splice columns of Msa", e.ruler)
            sys.exit(0)

# Try to keep track of original column numbering in remaining sequences
        if self.backtrack: # splcie existing
            backtrack = self.backtrack # just get ref, matrix wont be modified
            backtrackTmp = [ x for ind, x in enumerate(backtrack) if _isRuledIn(ind, seqRuler)]
            backtrackNew = [[ x for ind, x in enumerate(seq) if _isRuledIn(ind, colRuler)] for seq in backtrackTmp]

        else : # Create it from scarch
            backtrackNew = [];
            n = 0;
            for seq in matrixTmp:
                seqPos = -1
                backtrackNew.append([]);
                for ind,x in enumerate(seq):
                    if x != "-":
                        seqPos = seqPos + 1
                    if _isRuledIn(ind, colRuler):
                    #print str(ind) + "is ruled in"
                        if x == "-":
                            backtrackNew[n].append(-1)
                        else:
                            backtrackNew[n].append(seqPos)
                n = n + 1;

        #print backtrackNew



        #for ind, x in enumerate(self.backtrack):
        #    print "sequence %5d %s" % (ind, x)

        #print backtrackNew
        bean = MsaBean(header = header, matrix = matrix, backtrack = backtrackNew)
       # newMsa = Msa(msaBean = bean, backtrack = self.backtrack)
        newMsa = Msa(msaBean = bean)
        #print newMsa.backtrack
        return newMsa

    def gapPurge (self, gapRatio = 0.5):
        nGap = float(self.nSeq) < gapRatio
        sliceList = [ str(eO[0]) + ':' + str(eO[0]) for eO in enumerate(self.frequency) if float(eO[1]['-']) / float(self.nSeq) <= float(gapRatio)]
        print("keeping", str(len(sliceList)), "/", str(self.length),\
              " columns w/ gap ratio < ", gapRatio)
        #print str(sliceList)

        delCol = []
        for eO in enumerate(self.frequency):
            if float(eO[1]['-']) / float(self.nSeq) > float(gapRatio):
                delCol.append(eO)

        print("Total number of del columns ", str(len(delCol)))


        msaObject = self.sliceTo(columnSlice = sliceList)
        return msaObject

    def __repr__(self):

        header = "Alignment dimensions = " + str(self.nSeq) + " sequences, " + str(self.length) + " columns\n"
        # pretty print the frequencies
        freqAsString = "nCol" + re.sub("(?P<name>\w{1})", "  " + '\g<name>' + "  ", self.alphabet) + "\n"
        for iCol in range (self.length):
            freqAsString += "  "
            for iChar in list (self.alphabet):
                freqAsString += "%5s" % (self.frequency[iCol][iChar])
            freqAsString += "\n"
        return  header + freqAsString
        #return str(self.asMatrix)

    def fastaDump(self, outputFile=None):
        if (outputFile):
            f = open(outputFile,'w')
            f.write(self.format('fasta'))
            f.close()
        else :
            return self.format('fasta')

    def asDict(self):
        backtrack = ''
        if self.backtrack:
            backtrack = self.backtrack

        return {
            'matrix' : self.asMatrix,
            'headers' : self.headers,
            'backtrack' : backtrack,
            'frequency' : self.frequency
        }

    def asJSON(self):
        dic = self.asDict()
        jsonString = json.JSONEncoder().encode(dic)
        return jsonString

    def shape(self) :
        return [self.nSeq, self.length]

    # retrieve a sequence in the msa, by index or regular expression
    # gap will be striped
    def recordLookup(self, num=None, regExp=None):
        def _strip(r):
            d = copy.deepcopy(r)
            if 'seq' in d:
                d['sequence'] = d['seq']
                d.pop("seq", None)
            string = ''.join(d['sequence'])
            d['sequence'] = string.replace('-', '', len(string))
            return d

        if num or num == 0:
            d = self[num]
            return [_strip(d)]

        if not num and not regExp:
            return [_strip(d) for d in self]

        hits = []
        if regExp:
            for r in self:
                x = _strip(r)
                m = re.search(regExp, x['sequence'])
                if m:
                    #print m
                    hits.append(x)
            return hits

    def __getitem__(self, k):
        return {'header' : self.headers[k], 'sequence' : ''.join(self.asMatrix[k]) }

    def format(self, formatType):
        if not formatType is 'fasta':
            print("unknown format Type \'", str(formatType), "\'")
            sys.exit()

        msaAsString = ''
        for eO in enumerate(self.asMatrix):
            record = SeqRecord(Seq(''.join(eO[1]),IUPAC.protein),
                            id=self.headers[eO[0]], name="", description="")
            msaAsString += record.format("fasta")

        return msaAsString


# Just count the sim/id  and coverage(non-gap) of each sequence related to master
    def masterCoverage(self):

        master = self[0]['sequence']

        qData = [seqPairScores(master, self[i]['sequence']) for i in range(1 , len(self) )]

        return qData
'''
    def _scan_parallel(self, n):
        num_worker_threads = n
        hit_buffer = []
        Needle=scoringFunctions.Needle(alphabet=self.alphabet)
        def worker():
            while True:
                i_num = q.get()
                pept = self.getPept(i_num)

                nw_scan = nw_custom.nw(gapOpen=-10, gapExtend=-0.5, matchScorer=Needle.fScore)
                ali = nw_scan.align(queryPeptide, pept)
                hit_buffer.append({ 'sim' : float(ali.simPct) ,'ali': ali, 'num' : i_num})
                q.task_done()

        q = Queue()
        for i in range(num_worker_threads):
            t = Thread(target=worker)
            t.daemon = True
            t.start()

        for i in range(0, self.nSeq):
            q.put(i)


        print '*** Main thread waiting ' + str(q.qsize())
        q.join()
        #print '*** Done'
        return hit_buffer
'''




import sys
import os
import errno

from io import StringIO as StringIO
import re
import os.path
from types import ModuleType
import stat
import tarfile


def nonblank_lines(f):
    for l in f:
        line = l.rstrip()
        if line:
            yield line

'''
Container w/ loading/dumping capabilities for tsv formated text-file
'''
class Wrapper(object):
    def __init__(self, stream, ofs='\t', Nskip=0, dataParser=None, headerParser=None):
        self.ofs = ofs
        self.keymap = []
        self.data = []
        self.Nskip = Nskip
        self.dataParser = dataParser
        self.headerParser = headerParser


        if hasattr(stream, 'read'):
            self._loadStream(stream)
        elif isinstance(stream, basestring):
            if os.path.isfile(stream):
                self._loadFile(stream)
            else :
                self._loadString(stream)
        elif isinstance(stream, list):
            self._loadList(stream)
        else:
            raise TypeError('Cant parse/load ' + str(stream) )

    def _loadStream(self, stream):
        for i in range(self.Nskip):
            stream.readline()
        if self.headerParser:
            self.keymap = self.headerParser(stream.readline().replace("\n", ""))
        else:
            self.keymap = stream.readline().replace("\n", "").split(self.ofs)

        if self.dataParser:
            self.data = [ self.dataParser(l) for l in nonblank_lines(stream) ]
        else:
            self.data = [ self._push(l) for l in nonblank_lines(stream) ]



    def _loadList(self, inputList):

        if self.headerParser:
            self.keymap = self.headerParser( inputList[self.Nskip].replace("\n", "") )
        else :
            self.keymap = inputList[self.Nskip].replace("\n", "").split(self.ofs)

        if self.dataParser:
            self.data = [ self.dataParser(l) for l in nonblank_lines(inputList[ (self.Nskip+1): ])]
        else :
            self.data = [ self._push(l) for l in nonblank_lines(inputList[ (self.Nskip+1): ])]


    def _loadFile(self, fPath):
        with open(fPath, 'r') as stream:
            self._loadStream(stream)

    def _loadString(self, inputRaw):
        l = inputRaw.split('\n')
        self._loadList(l)


    def __len__(self):
        return len(self.data)

    def _push(self, inputRaw):
        #print ">" + self.ofs+ "<"
        #print inputRaw
        d = { self.keymap[i] : val for i, val in enumerate( inputRaw.split(self.ofs) ) }
        return d

    def __iter__(self):
        for d in self.data:
            yield d

    def __str__(self):
        asString= ''
        for i,value in enumerate(self.data):
            asString += '# Element n_' + str(i) + '\n'
            for k in self.keymap:
                if k not in value:
                    continue
                asString += '\'' + k + '\' : ' + value[ k ] + '\n'
        return asString

    def write(self):
        asString= '\t'.join(self.keymap) + '\n'
        for d in self :
            buf = [ d[ k ] for k in self.keymap ]
            asString += '\t'.join(buf) + '\n'
        return asString



import os

'''
GL 2015-07-08 Givin up trying to adpat this module to Peptides
 May come back to it later,if Peptide usage is needed on a long term
indexer arguments supply a indexing function to be called for each objects
    -->(typically setting object accessors key based on their fileName)
cIndexer arguments supply a common indexing function to be called once for the entiere collection
    --> eg: Read from a single catalogue file the mapping between object accessors and their fileName
'''
class EntrySet(object):
    def __init__(self, collectionPath=None, constructor=None, typeCheck=None, indexer=None, cIndexer=None):

        if not collectionPath:
            raise ValueError("You must specify a filesystem path to local cache")
        if not constructor:
            raise ValueError("You must bind an object constructor at cache invocation")
        if not indexer and not cIndexer:
            raise ValueError("You must specify an indexer function to map filename <-> collection key")

        self.typeCheck = typeCheck
        self.constructor = constructor
        self.indexer = indexer

        self.root = collectionPath
        self.data = {}
        # Try to index files
        self._index()

# Change cache location, re-index
    def setCache(self, location=None, reIndex=True):
        self.root = location
        print ("Changing cache location to " + self.root)
        if reIndex:
            print ("Reindexing " + self.root)
            self._index()

    def _index(self):
        for fileName in os.listdir(self.root): # lazy initialization of objects
            id = self.indexer(fileName)
            if id:
                self.data[id] = { 'updated' : False, 'location' : self.root + '/' + fileName, 'e' : None }  #Entry(id,fileName = self.root + '/' + fileName
        print ("Acknowledged " + str(len(self.data)) + " entries (" +  self.root + ")")

    def __len__(self):
        return len(self.data)

    def __iter__(self):
        for k,d in self.data.items():
            yield d

    def __repr__(self):
        string = "Cache pool size " + str(len(self)) + '\n'
        for k,d in self.data.items():
            if not k or not d:
                raise TypeError('Error here ::-> ' + str(k) + ' // ' + str(d) )
            status = "LOADED" if d['e'] else "LAZY"
            string += str(k) + '\t' + str(status) + "\n"
        return string

    def add(self, id, force=False):
        if not isinstance(id, str):
            for i in id:
                if not force and i in self.data:
    #                print str(i) + ' already part of collection'
                    continue
                self.data[i] = { 'updated' : True, 'location' : None, 'e' : self.constructor(i) } # Newly added object dont have location
        else :
            if not force and id in self.data:
    #            print str(i) + ' already part of collection'
                return
            self.data[id] = { 'updated' : True, 'location' : None, 'e' : self.constructor(id) } # Newly added object dont have location

    def clear(self):
        n=len(self.data)
        id=[ k for k in self.data ]
        for k in id:
            self.delete(k)
        print ("All " + str(n) + " entries deleted!")

    def delete(self, id):
        if os.path.isfile(self.data[id]['location']):
            os.remove(self.data[id]['location'])
        self.data.pop(id, None)

    def get(self, id, reload=False):
        if self.typeCheck:
            if not self.typeCheck(id):
                raise TypeError("invalid supplied identifier \"" + id + "\"")

        if id in self.data and reload:
            print ("Erasing prev ref")
            self.delete(id)
        # Present but lazy initialized
        if id in self.data and not self.data[id]['e'] and self.data[id]['location']:
            self.data[id]['e'] = self.constructor(id, fileName = self.data[id]['location'])
            self.data[id]['updated'] = True
        # Not present
        if not id in self.data:
            self.add(id)


        return self.data[id]['e']

    def serialize(self, force=False):
        c = 0
        t = 0
        for d in self:
            t += 1

            if not d['e']: # LAZY instantiated object, we skip it
                continue

            if d['updated'] or force:
                if d['e']:
                    d['location'] = self.root + '/' + d['e'].id + '.xml' if not d['location'] else d['location']

                mode = "wb"
                try:
                    data = d['e'].serialize().decode()
                    print(d['e'].id + ' has byte content')
                except AttributeError:
                    print(d['e'].id + ' has non byte content')
                    #print(d['e'].serialize())
                    mode = "w"

                with open(d['location'], mode) as textFile:
                    textFile.write(d['e'].serialize())
                c +=1
                d['updated'] = False
        print (str(c) + " entries updated, total pool is " + str(t))





def make_tarfile(output_filename, source_dir):
    with tarfile.open(output_filename, "w:gz") as tar:
        tar.add(source_dir, arcname=os.path.basename(source_dir))

# from http://stackoverflow.com/questions/12791997/how-do-you-do-a-simple-chmod-x-from-within-python
def chmodX(filePath):
    st = os.stat(filePath)
    os.chmod(filePath, st.st_mode | stat.S_IEXEC)

def checkEnv(keyList):
    for key in keyList:
        if not os.environ.get(key):
            raise ValueError("Environment variable " + key + " not defined")
    return True

# emulates find unix command
def find(sType='file', **kwargs):

    patt = kwargs['name'].replace("*", ".*")
    if 'path' not in kwargs and 'name' not in kwargs:
        raise ValueError
    if sType == 'file':
        result = [os.path.join(dp, f) for dp, dn, filenames in os.walk(kwargs['path']) for f in filenames if re.match(patt, f)]

    if sType == 'dir':
        result = [os.path.join(dp, d) for dp, dn, filenames in os.walk(kwargs['path']) for d in dn if re.match(patt, d)]

    return result


def which(program):
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file

    return None

def mkdir(path):
    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise



#import customCollection

import pyproteins.utils.unixCmd
import pyproteins.sequence.msa
import pyproteins.sequence.psipred

import os

import json
import random
import re
import collections
import uuid
#from Bio.Blast import NCBIXML
from subprocess import call
from pyproteins.utils.miscellaneous import lFormat
from Bio import SeqIO

aminoAcidTable = {
    'A' : ['ALA'],
    'C' : ['CYS'],
    'D' : ['ASP'],
    'E' : ['GLU'],
    'F' : ['PHE'],
    'G' : ['GLY'],
    'H' : ['HIS'],
    'I' : ['ILE'],
    'K' : ['LYS'],
    'L' : ['LEU'],
    'M' : ['MET', 'MSE'],
    'N' : ['ASN'],
    'P' : ['PRO'],
    'Q' : ['GLN'],
    'R' : ['ARG'],
    'S' : ['SER'],
    'T' : ['THR'],
    'V' : ['VAL'],
    'W' : ['TRP', 'TRY'],
    'Y' : ['TYR'],
    'X' : ['UNK']
}

def threeToOne(aa):
    for l in aminoAcidTable:
        if aa in  aminoAcidTable[l]:
            return l;
    print("Warning 3 letter code", aa, "not found")
    return 'X';

def oneToThree(aa):
    if aa in aminoAcidTable:
        return aminoAcidTable[aa][0]
    print("Warning one letter code", aa, "not found")
    return 'UNK';

class EntrySet(object): #customCollection.EntrySet
    def __init__(self, name=None, dataFile=None):
        self.data = collections.OrderedDict()
        #customCollection.EntrySet.__init__(self, collectionPath="/Users/guillaumelaunay/work/data/peptides", constructor=Entry, typeCheck=isValidID, indexer=strip)
        if not dataFile and not name:
            print("flat file storage missing, Can't creating empty set w/out a name")
            return
        if name:
            self.name = name
            return
        self._parse(dataFile)


    def pluck(self):
        k = random.choice(self.data.keys())
        return self.data[k]

    def mash(self):
        m = hash(self.name + str(len(self)))
        return str(abs(m))

    def _parse(self, dataFile):
        f = open(dataFile, "r")
        dBuffer = json.load(f)
        self.name = dBuffer['id']

        for d in dBuffer['data']:
            e = Entry(datum=d)
            if hash(e) in self.data: # hash string more efficient
                print("Mutliple definition of petide fragment \"", d['seq'], "\"")
            self.data[hash(e)] = e

    def __len__(self):
        return len(self.data)
    def __repr__(self):
        string = "Peptide set \"" + self.name + "\" ("  + str(len(self)) + " elements)\n"
        for e in self:
            string += e.id + "\n"
        return string

    def enrich(self, _blankShotID=None, service="arwen", *kwargs):
        pass
        #global ppServ
        #if not ppServ:
        #    ppServ = startPpServ(service=service)
        #stash = [d for d in self if not d.ss2]
        #jobid = ppServ.push(peptidesList=stash, _blankShotID=_blankShotID)
        #data = ppServ.pull(jobid)

        #for i,d in enumerate(stash):
            #d.ss2Bind(ss2Obj=data[i])
         #   datum = ppServ.pull(jobid)
         #   d.ss2Bind(ss2Obj=datum)

    def __iter__(self):
        for k in self.data:
            yield self.data[k]

    def __getitem__(self, k):
        for x in self.data:
            if self.data[x].id == k:
                return self.data[x]
        return None

    def index(self, string=None, peptideObject=None):
        if string:
            try:
                return [ e.id for e in self.data.values() ].index(string)
            except ValueError:
                return None
        if peptideObject:
            return [ e.id for e in self.data.values() ].index(peptideObject.id)


    def get(self, peptideObject):
        k = hash(peptideObject)
        if k in self.data:
            return self.data[k]
        return None

    def add(self, peptideObject):
        if self.delete(peptideObject):
            print("Following peptide already part of the set")
            print(peptideObject)
        self.data[hash(peptideObject)] = peptideObject

    def delete(self, peptideObject):
        k = hash(peptideObject)
        if k in self.data:
            del self.data[k]
            return True
        return False

    def serialize(self, targetFile=None, comments="peptide collection"):
        dataOut = {'id' : self.name, 'comments' : comments, 'data' : [ e.dict for e in self ]}
        asJson =  json.JSONEncoder().encode(dataOut)
        if targetFile:
            with open(targetFile, "w") as f:
                f.write(asJson)
        return asJson

    # Create a list of blast
    #
#blastBean : {
#    'env' : 'sge', 'slurm', 'local'[DEFAULT]
#    'blastDb' : REQUIRED
#    'BLASTDBROOT' : REQUIRED
#    'rootDir' : oc.getcwd() [DEFAULT]
#}
#

    def blastAll(self, blastBean, blastXmlOnly=False):
        pass


    @property # return list of peptides name
    def labels(self):
        return [ e.id for e in self ]




class Entry(object):

    def __getstate__(self): return self.__dict__
    def __setstate__(self, d): self.__dict__.update(d)

    def parse(self, fastaFile):
        handle = open(fastaFile, "rU")
        records = list(SeqIO.parse(handle, "fasta"))
        handle.close()
        self.id = records[0].id  #first record
        self.seq = str(records[0].seq)  #first record


    def __eq__(self, other):
        return self.seq == other.seq
            
    def __hash__(self):
        if not self.seed:
            random.seed()
            #self.seed = os.urandom(5)
            self.seed = random.randint(1,1000000)
        return hash(self.seq + str(self.seed))

    @property
    def tag(self):
        tag = re.sub('[\s]+','_', self.id)
        return tag

    def fastaWrite(self, path="./", fileName='default'):
        path = re.sub('[/]+$', '', path)
        fName = path + '/' + str(fileName) + '.fasta'
        with open (fName, 'w') as fOut:
                fOut.write(self.fasta)

    
        #print 'peptide \'' + self.id + '\' fasta wrote to file ' + fName


    def mash(self):
        m = hash(self)
        return str(abs(m))

    def serialize(self):
        return json.JSONEncoder().encode(self.dict) # Check that function name does not create None attribute
        pass # write json formated data structure call be customCollection

    def __init__(self, datum=None, **kwargs):
        self.description = None
        self.ss2Obj = None
        self.ss2Seq = None
        self.seq = None
        self.id = None
        self.seed = None # we need a random generator, bc sequence could be identical

        if datum:
            if 'id' not in datum or 'seq' not in datum:
                raise ValueError("Attribute missing for peptide definition")
            self.id = datum['id']
            self.seq = datum['seq']
            self.description = datum['desc']
            #if 'ss2' in datum:
            #    self.ss2Obj = pyproteins.services.psipredServ.collection(stream=datum['ss2'])
        else:
            if 'id' in kwargs:
                self.id = kwargs['id']
            if 'seq' in kwargs:
                self.seq = kwargs['seq']
            if 'desc' in kwargs:
                self.description = kwargs['desc']


       #     if 'ss2' in kwargs:
       #         self.ss2 = kwargs['ss2']

        #if not self.seq or not self.id:
        #    raise ValueError("Cant create peptide object")


    def __iter__(self):
        for i in range (0, len(self.seq)):
            yield self[i + 1]

    def __getitem__(self, i):
        if i < 0:
            raise ValueError("minimal amino-acid number is 1")
        v = {'aa' : self.seq[i], 'ss2' : None, 'burial' : None }
        if self.ss2:
            v['ss2'] = self.ss2[i]
        return pyproteins.sequence.msa.Position(v)

    @property
    def hasSse(self):
        if self.ss2:
            return True
        return False

    def __len__(self):
        return len(self.seq)

    def __repr__(self):
        s = self.fasta
        if self.ss2Obj:
            s += "\n" + self.ss2Obj.horiz
        return s
    @property
    def dict(self):
        ss2 = None
        if self.ss2Obj:
            ss2 = self.ss2Obj.dict
        return {
            'id' : self.id,
            'seq' : self.seq,
            'desc' : self.description,
            'ss2' : ss2
        }

    def ss2Bind(self, **kwargs):
       raise ValueError("NO SS2BIND")
       # ss2Obj = None
       # if 'ss2Obj' in kwargs:
       #     ss2Obj = kwargs['ss2Obj']
       # elif 'file' in kwargs:
       #     ss2Obj = pyproteins.utils.psipredServ.collection(fileName=kwargs['file'])

        #if ss2Obj.aaSeq != self.aaSeq:
        #    raise ValueError("amino-acid sequence dont match\nss2Obj_aaSeq : \"" + ss2Obj.aaSeq + "\"\n not equal to \npeptide_aaSeq : \"" + self.aaSeq + "\"")
        #self.ss2Obj = ss2Obj

    @property
    def aaSeq(self):
        return self.seq.upper()

    @property
    def fasta(self):
        add = ''
        if self.description:
            add = self.description
        return ">" + self.id + add + "\n" + lFormat(self.aaSeq)

    @property
    def ss2(self):
        if self.ss2Obj:
            return self.ss2Obj.horiz
        if self.ss2Seq:
            return self.ss2Seq
        return None
        #else:
        #    tmp = ppServ.push(peptidesList=[self])
        #    return tmp

    @property # try to extract pfam id
    def pfamID(self):
        m = re.search("(PF[\d]+)", self.id)
        if m:
            return m.groups(1)[0]
        return None

''' MUST REWRITE USING ETREE in dedicated parser
    def blast(self, xmlPsiBlastOutput=None, xmlPsiBlastStream=None, strict=False): # return an msa Object

        def hit_check(hit):
            for i in range(0, len(hit.hsps) - 1):
                for j in range(i + 1, len(hit.hsps)):
                    if hit.hsps[i].sbjct_start >= hit.hsps[j].sbjct_start:
                        if hit.hsps[i].sbjct_end <= hit.hsps[j].sbjct_end:
                            errorString =  "hsp  " + str(i) + " / " + str(j) + "overlap error in hit named " + hit.title + "\n"
                            errorString += str(hit.hsps[i].sbjct_start) + ',' + str(hit.hsps[i].sbjct_end) + ") vs (" + str(hit.hsps[j].sbjct_start) + ',' + str(hit.hsps[j].sbjct_end) + ')'
                            #raise ValueError( errorString)
                            print("Warning : " + errorString)

        def HspOverlap(mem, x):
            for y in mem:
                if (x[0] >= y[0] and  x[0] <= y[1]) : return True
                if (x[1] >= y[0] and  x[1] <= y[1]) : return True
            return False

        def hsp_merge(hsp, array):

            pos = hsp.query_start - 1
            for i in range(0, hsp.align_length):
                if hsp.query[i] == '-':
                    continue
                array[pos] = hsp.sbjct[i]
                pos += 1
            #return ''.join(mergedStrand)


        array=[{ 'id' : self.id,  'seq' : list(self.aaSeq) }]

        if not unixCmd.which("blastpgp"):
            raise initError("Cant find blastpgp executable")

        if not xmlPsiBlastOutput and not xmlPsiBlastStream:
            rootId = uuid.uuid4()
            self.fastaWrite(name=rootId)

            call(['blastpgp', '-j', '2', '-i', str(rootId) + '.fasta', '-d', 'nr70', '-m', '7', '-o', str(rootId) + '.xml'])
            f = open(str(rootId) + '.xml')
            records = NCBIXML.parse(f)
        else :
            if xmlPsiBlastOutput :
                f = open(xmlPsiBlastOutput)
            else:
                f = xmlPsiBlastStream
            records = NCBIXML.parse(f)

        recordList = [x for x in records]
        for psiPass in reversed(recordList):
            if not psiPass.alignments:
                continue
            for hit in psiPass.alignments:
                if len(hit.hsps) > 1:
                    if strict:
                        hit_check(hit)
                if hit.hsps[0].sbjct == self.aaSeq:
                    #print "Self hit, skipping"
                    continue
                title = str(hit.title) + '[' + ','.join([ str(hsp.sbjct_start) + '-' + str(hsp.sbjct_end) for hsp in hit.hsps ]) + ']'
                datum = { 'id' : title,  'seq' : ['-'] * len(self) }
                mem = []
                for hsp in hit.hsps:
                    if HspOverlap(mem, (hsp.sbjct_start,hsp.sbjct_end)):
                        if strict:
                            raise ValueError('hsp overlap error')
                        continue
                    mem.append((hsp.sbjct_start,hsp.sbjct_end))
                    hsp_merge(hsp, datum['seq'])
                array.append(datum)
            break

        f.close()
        #return [[ hit['seq'] for hit in array ], [ hit['id'] for hit in array ]]
        msaBean = pyproteins.sequence.msa.MsaBean([ hit['seq'] for hit in array ], [ hit['id'] for hit in array ])
        msa = pyproteins.sequence.msa.Msa(msaBean=msaBean)
        msa.set_id(self.id)
        #return msaBean
        return msa
'''


""" Module that monkey-patches json module when it's imported so
JSONEncoder.default() automatically checks for a special "toJSON()"
method and uses it to encode the object if found.
"""
from json import JSONEncoder

def _default(self, obj):
    return getattr(obj.__class__, "toJSON", _default.default)(obj)

_default.default = JSONEncoder().default  # Save unmodified default.
JSONEncoder.default = _default # Replace it.


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import, print_function

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup

setup(
  name = 'pyproteins',
  version = '1.0',
  license='BSD',
  description = 'Toolbox to manipulate protein sequence data',
  author = 'Guillaume Launay',
  author_email = 'pitooon@gmail.com',
  url = 'https://github.com/glaunay/pyproteins', # use the URL to the github repo
  packages=find_packages('src'),
  package_dir={'': 'src'},
  include_package_data=True,
  zip_safe=False,
  py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
  download_url = 'https://github.com/glaunay/pyproteins/tarball/1.0', # I'll explain this in a second
  keywords = ['protein', 'sequence'], # arbitrary keywords
  classifiers = [],
  install_requires=[
          'bs4', 'biopython', 'numpy','lxml'
      ],
   package_data = {
   'pyproteins': ['conf/confModule1.json','bin/module1.py', 'external/*']
   },
  #data_files=[
  #          ('external', ['external/pathos.tar.bz']),
  #          ('bin', ['bin/module1.py']),
  #          ('conf',['conf/confModule1.json'])
  #    ]
  #  dependency_links = [
  #      "http://dev.danse.us/trac/pathos"
  #  ]
)



import os
import re

# Not intended to make psipred calls
# Simple parser



# for now we only parse the horiz file
def parse(folder=None):
    data = {'Conf':'', 'Pred':'', 'AA':''}

    for file in os.listdir(folder):
        if file.endswith("horiz"):
            print(folder + '/' + file)
            with open(folder + '/' + file, 'r') as f:
                for l in f:
                    m = re.match('^[\s]*(Conf|Pred|AA):[\s]+([\S]+)[\s]*$', l)
                    if m:
                        data[m.group(1)] += m.group(2)

    return Container(conf=data['Conf'], pred=data['Pred'], aa=data['AA'])


class Container(object):
    def __init__(self, **kwargs):
        self.regKeys = ['conf', 'pred', 'aa']
        for key in self.regKeys:
            if key in kwargs:
                setattr(self, key, kwargs[key])

    def __repr__(self):
        string = ''
        for key in self.regKeys:
            if hasattr(self, key):
                string += key + ':' + getattr(self, key) + '\n'
        return string
